@@ -28,6 +28,7 @@ modules = [
     "blogger",
     "bunkr",
     "catbox",
+    "chevereto",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -73,7 +74,6 @@ modules = [
     "issuu",
     "itaku",
     "itchio",
-    "jpgfish",
     "jschan",
     "kabeuchi",
     "keenspot",
