@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2022 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,18 +9,17 @@
 """Extractors for https://www.patreon.com/"""
 
 from .common import Extractor, Message
-from .. import text, exception
+from .. import text, util, exception
 from ..cache import memcache
 import collections
 import itertools
-import json
 
 
 class PatreonExtractor(Extractor):
     """Base class for patreon extractors"""
     category = "patreon"
     root = "https://www.patreon.com"
-    cookiedomain = ".patreon.com"
+    cookies_domain = ".patreon.com"
     directory_fmt = ("{category}", "{creator[full_name]}")
     filename_fmt = "{id}_{title}_{num:>02}.{extension}"
     archive_fmt = "{id}_{num}"
@@ -29,11 +28,11 @@ class PatreonExtractor(Extractor):
     _warning = True
 
     def items(self):
-
         if self._warning:
-            if not self._check_cookies(("session_id",)):
+            if not self.cookies_check(("session_id",)):
                 self.log.warning("no 'session_id' cookie set")
             PatreonExtractor._warning = False
+
         generators = self._build_file_generators(self.config("files"))
 
         for post in self.posts():
@@ -95,7 +94,7 @@ class PatreonExtractor(Extractor):
         if content:
             for img in text.extract_iter(
                     content, '<img data-media-id="', '>'):
-                url = text.extract(img, 'src="', '"')[0]
+                url = text.extr(img, 'src="', '"')
                 if url:
                     yield "content", url, self._filename(url) or url
 
@@ -103,7 +102,9 @@ class PatreonExtractor(Extractor):
         """Return all relevant post objects"""
 
     def _pagination(self, url):
-        headers = {"Referer": self.root}
+        headers = {
+            "Content-Type": "application/vnd.api+json",
+        }
 
         while url:
             url = text.ensure_http_scheme(url)
@@ -181,7 +182,7 @@ class PatreonExtractor(Extractor):
         """Fetch filename from an URL's Content-Disposition header"""
         response = self.request(url, method="HEAD", fatal=False)
         cd = response.headers.get("Content-Disposition")
-        return text.extract(cd, 'filename="', '"')[0]
+        return text.extr(cd, 'filename="', '"')
 
     @staticmethod
     def _filehash(url):
@@ -199,23 +200,36 @@ class PatreonExtractor(Extractor):
         return (
             "https://www.patreon.com/api/" + endpoint +
 
-            "?include=user,images,attachments,user_defined_tags,campaign,poll."
-            "choices,poll.current_user_responses.user,poll.current_user_respon"
-            "ses.choice,poll.current_user_responses.poll,access_rules.tier.nul"
-            "l"
+            "?include=campaign,access_rules,attachments,audio,images,media,"
+            "native_video_insights,poll.choices,"
+            "poll.current_user_responses.user,"
+            "poll.current_user_responses.choice,"
+            "poll.current_user_responses.poll,"
+            "user,user_defined_tags,ti_checks"
 
-            "&fields[post]=change_visibility_at,comment_count,content,current_"
-            "user_can_delete,current_user_can_view,current_user_has_liked,embe"
-            "d,image,is_paid,like_count,min_cents_pledged_to_view,post_file,pu"
-            "blished_at,patron_count,patreon_url,post_type,pledge_url,thumbnai"
-            "l_url,teaser_text,title,upgrade_url,url,was_posted_by_campaign_ow"
-            "ner"
+            "&fields[campaign]=currency,show_audio_post_download_links,"
+            "avatar_photo_url,avatar_photo_image_urls,earnings_visibility,"
+            "is_nsfw,is_monthly,name,url"
+
+            "&fields[post]=change_visibility_at,comment_count,commenter_count,"
+            "content,current_user_can_comment,current_user_can_delete,"
+            "current_user_can_view,current_user_has_liked,embed,image,"
+            "insights_last_updated_at,is_paid,like_count,meta_image_url,"
+            "min_cents_pledged_to_view,post_file,post_metadata,published_at,"
+            "patreon_url,post_type,pledge_url,preview_asset_type,thumbnail,"
+            "thumbnail_url,teaser_text,title,upgrade_url,url,"
+            "was_posted_by_campaign_owner,has_ti_violation,moderation_status,"
+            "post_level_suspension_removal_date,pls_one_liners_by_category,"
+            "video_preview,view_count"
+
+            "&fields[post_tag]=tag_type,value"
             "&fields[user]=image_url,full_name,url"
-            "&fields[campaign]=avatar_photo_url,earnings_visibility,is_nsfw,is"
-            "_monthly,name,url"
-            "&fields[access_rule]=access_rule_type,amount_cents" + query +
+            "&fields[access_rule]=access_rule_type,amount_cents"
+            "&fields[media]=id,image_urls,download_url,metadata,file_name"
+            "&fields[native_video_insights]=average_view_duration,"
+            "average_view_pct,has_preview,id,last_updated_at,num_views,"
+            "preview_views,video_duration" + query +
 
-            "&json-api-use-default-includes=false"
             "&json-api-version=1.0"
         )
 
@@ -234,6 +248,10 @@ class PatreonExtractor(Extractor):
             filetypes = filetypes.split(",")
         return [genmap[ft] for ft in filetypes]
 
+    def _extract_bootstrap(self, page):
+        return util.json_loads(text.extr(
+            page, "window.patreon.bootstrap,", "});") + "}")
+
 
 class PatreonCreatorExtractor(PatreonExtractor):
     """Extractor for a creator's works"""
@@ -241,34 +259,7 @@ class PatreonCreatorExtractor(PatreonExtractor):
     pattern = (r"(?:https?://)?(?:www\.)?patreon\.com"
                r"/(?!(?:home|join|posts|login|signup)(?:$|[/?#]))"
                r"([^/?#]+)(?:/posts)?/?(?:\?([^#]+))?")
-    test = (
-        ("https://www.patreon.com/koveliana", {
-            "range": "1-25",
-            "count": ">= 25",
-            "keyword": {
-                "attachments"  : list,
-                "comment_count": int,
-                "content"      : str,
-                "creator"      : dict,
-                "date"         : "type:datetime",
-                "id"           : int,
-                "images"       : list,
-                "like_count"   : int,
-                "post_type"    : str,
-                "published_at" : str,
-                "title"        : str,
-            },
-        }),
-        ("https://www.patreon.com/koveliana/posts?filters[month]=2020-3", {
-            "count": 1,
-            "keyword": {"date": "dt:2020-03-30 21:21:44"},
-        }),
-        ("https://www.patreon.com/kovelianot", {
-            "exception": exception.NotFoundError,
-        }),
-        ("https://www.patreon.com/user?u=2931440"),
-        ("https://www.patreon.com/user/posts/?u=2931440"),
-    )
+    example = "https://www.patreon.com/USER"
 
     def __init__(self, match):
         PatreonExtractor.__init__(self, match)
@@ -282,10 +273,12 @@ class PatreonCreatorExtractor(PatreonExtractor):
             url = "{}/user/posts?u={}".format(self.root, creator_id)
         else:
             url = "{}/{}/posts".format(self.root, self.creator)
-
         page = self.request(url, notfound="creator").text
-        campaign_id = text.extract(page, "/campaign/", "/")[0]
-        if not campaign_id:
+
+        try:
+            data = self._extract_bootstrap(page)
+            campaign_id = data["campaign"]["data"]["id"]
+        except (KeyError, ValueError):
             raise exception.NotFoundError("creator")
 
         filters = "".join(
@@ -295,10 +288,10 @@ class PatreonCreatorExtractor(PatreonExtractor):
         )
 
         url = self._build_url("posts", (
-            "&sort=" + query.get("sort", "-published_at") +
-            "&filter[is_draft]=false"
+            "&filter[campaign_id]=" + campaign_id +
             "&filter[contains_exclusive_posts]=true"
-            "&filter[campaign_id]=" + campaign_id + filters
+            "&filter[is_draft]=false" + filters +
+            "&sort=" + query.get("sort", "-published_at")
         ))
         return self._pagination(url)
 
@@ -307,12 +300,13 @@ class PatreonUserExtractor(PatreonExtractor):
     """Extractor for media from creators supported by you"""
     subcategory = "user"
     pattern = r"(?:https?://)?(?:www\.)?patreon\.com/home$"
-    test = ("https://www.patreon.com/home",)
+    example = "https://www.patreon.com/home"
 
     def posts(self):
         url = self._build_url("stream", (
             "&page[cursor]=null"
             "&filter[is_following]=true"
+            "&json-api-use-default-includes=false"
         ))
         return self._pagination(url)
 
@@ -321,24 +315,7 @@ class PatreonPostExtractor(PatreonExtractor):
     """Extractor for media from a single post"""
     subcategory = "post"
     pattern = r"(?:https?://)?(?:www\.)?patreon\.com/posts/([^/?#]+)"
-    test = (
-        # postfile + attachments
-        ("https://www.patreon.com/posts/precious-metal-23563293", {
-            "count": 4,
-        }),
-        # postfile + content
-        ("https://www.patreon.com/posts/56127163", {
-            "count": 3,
-            "keyword": {"filename": r"re:^(?!1).+$"},
-        }),
-        # tags (#1539)
-        ("https://www.patreon.com/posts/free-post-12497641", {
-            "keyword": {"tags": ["AWMedia"]},
-        }),
-        ("https://www.patreon.com/posts/not-found-123", {
-            "exception": exception.NotFoundError,
-        }),
-    )
+    example = "https://www.patreon.com/posts/TITLE-12345"
 
     def __init__(self, match):
         PatreonExtractor.__init__(self, match)
@@ -347,8 +324,7 @@ class PatreonPostExtractor(PatreonExtractor):
     def posts(self):
         url = "{}/posts/{}".format(self.root, self.slug)
         page = self.request(url, notfound="post").text
-        data = text.extract(page, "window.patreon.bootstrap,", "\n});")[0]
-        post = json.loads(data + "}")["post"]
+        post = self._extract_bootstrap(page)["post"]
 
         included = self._transform(post["included"])
         return (self._process(post["data"], included),)
