@@ -125,7 +125,8 @@ class RedditExtractor(Extractor):
                     if match:
                         extra.append(match.group(1))
                     elif not match_user(url) and not match_subreddit(url):
-                        if previews and "preview" in data:
+                        if previews and "comment" not in data and \
+                                "preview" in data:
                             data["_fallback"] = self._previews(data)
                         yield Message.Queue, text.unescape(url), data
                         if "_fallback" in data:
